package Form::Processor::Field::DateTime;
use strict;
use warnings;
use base 'Form::Processor::Field::DateTimeManip';

our $VERSION = '0.01';

=head1 NAME

Form::Processor::Field::DateTime

=head1 DESCRIPTION

This is a module that allows mapping the DateTime field to
the actual field used.  Makes it easier to remap all the forms
in an application to a new field type by overriding instead of
editing all the forms.

=cut


1;

