package Form::Processor::Field::Phone;
use strict;
use warnings;
use base 'Form::Processor::Field::Text';


our $VERSION = '0.01';

# This is a placeholder which is just a text field
# which can later be updated to a multifield area/prefix/num/extension
# field


1;

